//
// System.Web.UI.DataBoundLiteralCOntrol.cs
//
// Authors:
// 	Duncan Mak  (duncan@ximian.com)
// 	Gonzalo Paniagua Javier (gonzalo@ximian.com)
//
// (C) 2002 Ximian, Inc. (http://www.ximian.com)
//

//
// Permission is hereby granted, free of charge, to any person obtaining
// a copy of this software and associated documentation files (the
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to
// permit persons to whom the Software is furnished to do so, subject to
// the following conditions:
// 
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//

using System;
using System.ComponentModel;
using System.Text;

namespace System.Web.UI {

	[ToolboxItem(false)]
	public sealed class DataBoundLiteralControl : Control
	{
		private string [] staticLiterals;
		private string [] dataBoundLiterals;
		
		public DataBoundLiteralControl (int staticLiteralsCount,
						int dataBoundLiteralCount)
		{
			staticLiterals = new string [staticLiteralsCount];
			dataBoundLiterals = new string [dataBoundLiteralCount];
			PreventAutoID ();
		}

		public string Text {
			get {
				StringBuilder text = new StringBuilder ();
				int stLength = staticLiterals.Length;
				int dbLength = dataBoundLiterals.Length;
				int max = (stLength > dbLength) ? stLength : dbLength;
				for (int i = 0; i < max; i++){
					if (i < stLength)
						text.Append (staticLiterals [i]);
					if (i < dbLength)
						text.Append (dataBoundLiterals [i]);
				}

				return text.ToString ();
			}
		}

		protected override ControlCollection CreateControlCollection ()
		{
			return new EmptyControlCollection (this);
		}

		protected override void LoadViewState (object savedState)
		{
			if (savedState != null) {
				Array source = (Array) savedState;
				if (source.Length == dataBoundLiterals.Length)
					source.CopyTo (dataBoundLiterals, 0);
			}
		}

		protected override void Render (HtmlTextWriter output)
		{
			output.Write (Text);
		}

		protected override object SaveViewState ()
		{
			if (dataBoundLiterals.Length == 0)
				return null;
			return dataBoundLiterals;
		}

		public void SetDataBoundString (int index, string s)
		{
			dataBoundLiterals [index] = s;
		}

		public void SetStaticString (int index, string s)
		{
			staticLiterals [index] = s;
		}
	}
}

