//
// System.Web.UI.ValidatorCollection.cs
//
// Authors:
// 	Duncan Mak  (duncan@ximian.com)
// 	Gonzalo Paniagua Javier (gonzalo@ximian.com)
//
// (C) 2002 Ximian, Inc. (http://www.ximian.com)

//
// Permission is hereby granted, free of charge, to any person obtaining
// a copy of this software and associated documentation files (the
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to
// permit persons to whom the Software is furnished to do so, subject to
// the following conditions:
// 
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//

using System;
using System.Collections;

namespace System.Web.UI {

	public sealed class ValidatorCollection : ICollection, IEnumerable
	{
		private ArrayList _validators;

		public ValidatorCollection ()
		{
			_validators = new ArrayList ();
		}

		public int Count {
			get { return _validators.Count; }
		}

		public bool IsReadOnly {
			get { return _validators.IsReadOnly; }
		}

		public bool IsSynchronized {
			get { return _validators.IsSynchronized; }
		}

		public IValidator this [int index] {
			get { return (IValidator) _validators [index]; }
		}

		public object SyncRoot {
			get { return this; }
		}

		public void Add (IValidator validator)
		{
			_validators.Add (validator);
		}

		public bool Contains (IValidator validator)
		{
			return _validators.Contains (validator);
		}

		public void CopyTo (Array array, int index)
		{
			_validators.CopyTo (array, index);
		}

		public IEnumerator GetEnumerator ()
		{
			return _validators.GetEnumerator ();
		}

		public void Remove (IValidator validator)
		{
			_validators.Remove (validator);
		}
	}
}
